## This is the plugin to handle smb laserwriters under
## tkchooser2. It has no access to global variables, only
## procedures and functions
##
## Ethan Gold <etgold@cs.vassar.edu> 2/24/98
##

## must provide a name, public name, and icon in addition to
## start and stop procedures

## these don't work as globals, only for
## non-procedural initialization

if {[debug]} { puts "loading smbshare..." }

## register the plugin with the main chooser
register_plugin smb smbshare "Shared Disks"

########## required functions #################

#### required procedures that simply return plugin information
proc smbshare.geticon {} { return "smbshare.pnm" }
proc smbshare.getpubname {} { return "Shared Disks" }
proc smbshare.getprotocol {} { return "smb" }
proc smbshare.afterdelay {} { return "120000" }

## start function - required
proc smbshare.start {} {
    if {[debug]} { puts "started Shared Disks Plugin" }

    set entlabel [plug_list_label]
    $entlabel configure -text "Machines:"
    update
    smbshare.widgets stop
    smbshare.widgets start
    smbshare.refresh
}

## stop funtion - required
proc smbshare.stop {} {
    if {[debug]} { puts "stopped Shared Disks Plugin" }
    smb.delmachines
    set ID [get_glob afterID]
    if {[string compare $ID ""] != 0 } {
	after cancel $ID
    }
    smbshare.widgets stop
}

## function to call when a listitem is double-clicked
proc smbshare.doubleclick {} {
    if {[debug]} { puts "smbshare: got double-click with [get_curr_item]" }
    set plugframe [plug_frame]
    $plugframe.mount invoke
}

proc smbshare.newzone {} {
    smb.setmachines
    #set plugframe [plug_frame]
    #$plugframe.status configure -text "scanning [smb.getcurrzone]..."
    #update
    #smbshare.clearentities
    #smbshare.showentities [smbshare.getnames]
    #$plugframe.status configure -text "ready."
}

############# End required functions ##################

## function to do tilde completion on a filename
proc smbshare.tilde {name} {
    global env
    ## do tilde expansion
    regsub "~/" $name "$env(HOME)/" newfile
    regsub $env(USER) $env(HOME) "" tophome
    regsub "~" $newfile $tophome file_name
    return $file_name
}

proc smbshare.refresh {} {
    smbshare.newzone
    set_glob afterID [ after [smbshare.afterdelay] {smbshare.refresh}]
}

## procedure to actually invoke the mounting commands
proc smbshare.mountvol {guestflag username password machine filename} {
    global smbdir

    set volume [get_glob volume]
    set smbmount $smbdir/smbmount
    set filename [smbshare.tilde $filename]

    ## make sure the needed parameters are set
    if {[string compare $volume ""] == 0} {return}
    if {[string compare $filename ""] == 0} {return}

    if {$guestflag} { 
	set name "-N"
    } elseif {[string compare $username ""] == 0} {
	set name "-N"
    } else { set name "$username" }


    if {[debug]} {
	puts "smbshare: mountvol called with $guestflag, \
		$username, $password, $filename, $volume" 
    }
	
    if {$guestflag} {
	puts "smbshare: mounting $volume as guest"
	catch {exec $smbmount //$machine/$volume $filename $name} result
    } else {
	if {[debug]} {puts "execing: $smbmount //$machine/$volume $filename -U $name -P \"$password\""}
	catch {exec $smbmount //$machine/$volume $filename -U $name -P $password} result
    }
    puts $result

}

## function to build window to mount volumes
proc smbshare.mountwindow {server} {
    
    set username ""
    set password ""
    ## make sure something's selected
    if {[string compare $server ""] == 0} {
	return
    }

    set w ".smbshare_mountwin"
    set guestflag 0

    ## we are only going to support one mount window
    ## at a time for now. this is easily changed.
    destroy $w
    
    ## build mounting window
    toplevel $w -class Dialog
    label $w.label -text \
	    " \nConnect to the file server \"$server\" as: \n" \
	    -font [get_propfont]

    checkbutton $w.guest -variable guestflag \
	    -text "Guest" -font [get_propfont]
    label $w.spacelabel1 -text "\n"
    label $w.spacelabel2 -text "\n"
    frame $w.nameframe
    frame $w.passframe
    label $w.nameframe.namelabel -text "Name:     " -font [get_fixedfont]
    label $w.passframe.passlabel -text "Password: " -font [get_fixedfont]
    entry $w.nameframe.name -textvariable username -exportselection 0
    entry $w.passframe.pass -textvariable password -exportselection 0 -show "*"
    pack $w.nameframe.namelabel -side left -anchor n
    pack $w.nameframe.name -side right -anchor n
    pack $w.passframe.passlabel -side left -anchor n
    pack $w.passframe.pass -side right -anchor n
    button $w.cancel -text "Cancel" \
	    -command "destroy $w"
    button $w.ok -text "OK" \
	    -command "destroy $w; smbshare.volwin \"\$guestflag\" \"\$username\" \"\$password\" \"$server\"" -default active

    pack $w.label -side top -padx 5
    pack $w.guest -side top -anchor w -padx 15
    pack $w.spacelabel1 -side top
    pack $w.nameframe -side top -anchor w  -padx 15
    pack $w.passframe -side top -anchor w  -padx 15
    #pack $w.spacelabel2 -side top
    pack $w.cancel -side left -anchor s -padx 10 -pady 10
    pack $w.ok -side right -anchor s -padx 10 -pady 10
    update
    focus $w.nameframe.name

    bind $w.passframe.pass <Return> "$w.ok invoke"
    bind $w <Control-c> "$w.cancel invoke"
    bind $w.cancel <Return> "$w.cancel invoke"
    bind $w.ok <Return> "$w.ok invoke"
}

## procedure to get a filename, set it globally, and list
## it in the filename widget
proc smbshare.getfilename {filename} {    
    set_glob file "[pwd]/$filename"
    .smbsharewin.fframe.file configure -text [get_glob "file"]
    if {[debug]} { puts "smbshare: file was [get_glob file]" }
}

## procedure to call smb mount function
proc smbshare.volwin {guestflag username password machine} {
    if {[debug]} {
	puts "smbshare: mountwin called with $guestflag, \
		$username, $password, $machine"
    }

    set_glob volume ""
    set_glob file ""
    set file ""

    ## build the window
    set w .smbsharewin
    destroy $w
    toplevel $w -class Dialog
    frame $w.lframe
    frame $w.fframe
    label $w.label -text "Volumes on $machine: " -relief groove
    listbox $w.lframe.volumes -yscrollcommand "$w.lframe.scroll set" \
	    -height 6 -width 30 -exportselection 0
    scrollbar $w.lframe.scroll -command "$w.lframe.volumes yview" -takefocus 0
    #message $w.fframe.file -width 200 -relief sunken -text ""
    entry $w.fframe.file -width 20 -relief sunken -textvariable file \
	    -exportselection 0
    label $w.fframe.label -text "Mount Point: "
    button $w.mount -text "Mount" -command \
	    "smbshare.mountvol $guestflag \"$username\" \"$password\" \"$machine\" \$file"
    button $w.cancel -text "Close" -command "destroy $w"
   
    pack $w.lframe.volumes -side left
    pack $w.lframe.scroll -side right -fill y
    pack $w.fframe.label -side left
    pack $w.fframe.file -side right

    pack $w.label -fill x
    pack $w.lframe
    pack $w.fframe
    pack $w.cancel -side left -anchor s -padx 10 -pady 10
    pack $w.mount -side right -anchor s -padx 10 -pady 10

    bind $w.fframe.file <Return> "$w.mount invoke"
    
    bind $w.lframe.volumes <ButtonRelease> \
	    "set_glob volume \[$w.lframe.volumes get \[$w.lframe.volumes curselection\]\]"

    ## fill the listbox with names
    set services [smb.getservices $machine "Disk" $guestflag $username $password]
    foreach name $services {
	$w.lframe.volumes insert end $name
    }
    if {[debug]} {puts "smb: got services: $services"}
}

## procedure to build and destroy smbshare widgets
proc smbshare.widgets {command} {
    set plugframe [plug_frame]
    
    if {[string compare $command "start"] == 0} {
	button $plugframe.mount -text "Mount..." \
		-command "smbshare.mountwindow  \"\[get_curr_item\]\""
	
	pack $plugframe.mount
	$plugframe.status configure -text "ready."

    } else {
	destroy $plugframe.mount
    }
}

## function to return the machines in the current zone
proc smbshare.getnames {} {
    return [smb.namesfromlkup [smbshare.getentities]]
}

proc smbshare.getentities {} {
    set currzone [smb.getcurrzone]
    set entity ""
    set results [smb.nbplkup $entity]
    set results [smb.namesfromlkup $results]
    return $results
}

## function to display passed in entities in
## the plugin's entity list frame
proc smbshare.showentities {entitylist} {
    set lbox [plug_list]
    foreach entity $entitylist {
	$lbox insert end $entity
    }
}

proc smbshare.clearentities {} {
    set lbox [plug_list]
    $lbox delete 0 end
}

if {[debug]} { puts "finished loading smbshare." }
