## This is the smb module for tkchooser2
## This file defines and provides a higher-level
## API to the underlying software layer (Netatalk, in this case)
## and handles non service-specific stuff
## Ideally a plugin relying on smb doesn't need to know
## what the underlying smb layer is built with. A
## CAP smb module could drop in and replace this one
## without the plugins caring at all.
##
## Major protocol modules must provide a start function,
## a setcurrzone function, and an unload function
##
## Ethan Gold <etgold@cs.vassar.edu> 2/24/98
##

if {[debug]} { puts "Loading samba smb module..." }

#### Smb global variables ####
## global "smbdir" defined in the config file
set smb_currzone ""
set smb_defzone ""
set smb_zonelist ""
set smb_zones(dummy) dum
unset smb_zones(dummy)

## delay between zonelist refreshed in milliseconds
## this doesn't need to happen very often
set smb_zonedelay 180000
set smb_afterID ""

## external program names


#### End Smb global variables ####

############ Required functions - API to main ##############

## startup procedure
proc smb.start {} {
    global smb_currzone smb_defzone
    if {[debug]} { puts "smb: starting smb module" }
    smb.delzones
    smb.delmachines
    smb.getdefzone
    smb.setcurrzone "$smb_defzone"
    smb.refresh
    smb.setdefzone
    #smb.setmachines
}

## shutdown procedure
proc smb.stop {} {
    global smb_afterID
    if {[debug]} { puts "smb: stopping smb module" }
    after cancel $smb_afterID
}

## procedure to set the current zone
proc smb.setcurrzone {newzone} {
    global smb_currzone
    set smb_currzone $newzone
    if {[debug]} {puts "smb: setting current zone to $newzone"}
}

############ End Required functions - API to main ##############
############ Functions plugins will need ##############

## function to return current zone
proc smb.getcurrzone {} {
    global smb_currzone
    return $smb_currzone
}

## function to lookup a given entity
proc smb.nbplkup {entity} {
    set results ""
    
    
    
    return $results
}

## function to return the current zone
proc smb.getcurrzone {} {
    global smb_currzone
    return $smb_currzone
}

## function to parse names out of entity lists
proc smb.namesfromlkup {results} {
    set names [list]
    return $names
}

## function to send a file to the printer using
## whatever the printer-talking utility is called.
## This MIGHT have to move into the plugin if all
## sorts of happy printer options are to be supported.
proc smb.print {printer filename} {
    global smbdir
    set result ""
    return $result
}

#########  End Functions plugins will need ############

######### begin general utility functions ###########
## function to return zones
proc smb.getzones {} {
    global smbdir env smb_zones
    set plugframe [plug_frame]
    set smb_zonelist ""
    set workgroup [smb.getdefzone]
    if {[info exists env(HOSTNAME)] == 1} {
	set hostname $env(HOSTNAME)
    } else {
	catch {exec hostname -s} hostname
    }
    set master $hostname
    set smb_zones($workgroup) $hostname
    if {[debug]} {puts "smb: getting workgroups..."}

    $plugframe.status configure -text "reading workgroups..."
    update

    ## search for the workgroup master
    set info [open "|echo \"\" | $smbdir/smbclient -N -L $hostname" r]
    while {[gets $info line] != -1} {
	if {[debug]} {puts "smb: $line"}
	if { [regexp "^\t($workgroup)\ +" $line] } {
	    if {[debug]} {puts "smb: workgroup master?: $line"}
	    set master [lindex $line 1]
	    regsub "^\t$workgroup\ (\[.| \]+)" $line {\1} master
	    set master [string trim $master]
	    if {[debug]} {puts "smb: master is $master"}
	    break
	}
    }
    catch {close $info}

    if {[debug]} {puts "smb: using $master for workgroup lookup"}

    ## open pipe to smbclient - echo in a null string
    ## in case it asks for a password
    ## just in case the idiot asks for a password
    set info [open "|echo \"\" | $smbdir/smbclient -N -L $master" r]
    while {[gets $info line] != -1} {
	if { [regexp {.*Workgroup.*Master.*} $line] } {
	    if {[debug]} {puts "smb: $line"}
	    ## eat the visual seperator line
	    gets $info $line
	    ## read the workgroups and masters
	    while {[gets $info line] != -1} {
		## idiotic unparseable smbclient formatting
		if {[debug]} {puts "$line"}
		set newmaster [string range $line 22 end]
		regsub -all "\t(.+)($newmaster)" $line {\1} newgrp
		set newgrp [string trim $newgrp]
		if {[debug]} {puts "smb: newgrp = $newgrp; newmaster = $newmaster"}
		set smb_zonelist "$smb_zonelist \"$newgrp\""
		## save the master for each zone
		set smb_zones($newgrp) \"$newmaster\"
	    }
	}
    }
    catch {close $info}

    $plugframe.status configure -text "ready."
    update
    if {[debug]} { puts "smb: workgroups: $smb_zonelist" }

    if {[debug]} {puts "smb: done getting workgroups."}
    return $smb_zonelist
}

## function to return the names of machines
## in a zone
proc smb.getmachines {workgroup} {
    global smb_zones smbdir smb_currzone

    set master $smb_zones($workgroup)
    set smb_machinelist ""
    set plugframe [plug_frame]
    if {[debug]} {puts "smb: getmachines: scanning $workgroup with $master"}

    $plugframe.status configure -text "scanning $smb_currzone..."
    update

    ## open pipe to smbclient - echo in a null string
    ## in case it asks for a password
    ## just in case the idiot asks for a password
    set info [open "|echo \"\" | $smbdir/smbclient -N -L $master" r]

    while {[gets $info line] != -1} {
	if { [regexp {.*Server.*Comment.*} $line] } {
	    if {[debug]} {puts "smb: $line"}
	    ## eat the visual seperator line
	    gets $info $line
	    ## read the servers in the workgroup
	    gets $info line
	    while {![regexp {.*machine has a workgroup list.*} $line]} {
		## idiotic unparseable smbclient formatting
		set comment [string range $line 22 end]
		regsub -all "\t(.+)($comment)" $line {\1} newmachine
		set newmachine [string trim $newmachine]
		if {[debug]} {
		    puts "smb: newmachine = $newmachine; comment = $comment" }
		set smb_machinelist "$smb_machinelist \"$newmachine\""
		if {[gets $info line] == -1} { break }
	    }
	}
    }
    catch {close $info}
    if {[debug]} {puts "smb: getmachines: $smb_machinelist"}
    $plugframe.status configure -text "ready."
    update
    return $smb_machinelist
}

## procedure to list the machine names
## in the plugin's listbox window. This
## is being done in the smb module because
## windows networks are machine oriented, not
## service oriented.
proc smb.setmachines {} {
    global smb_currzone
    set pluglist [plug_list]

    smb.delmachines
    set machines [smb.getmachines "$smb_currzone"]
    foreach machine $machines {
	if {[string compare $machine ""] != 0} {
	    $pluglist insert end $machine
	}
    }
    update
}

proc smb.delmachines {} {    
    set pluglist [plug_list]
    $pluglist delete 0 end
}


## function to return a list of services of a
## given type for a given machine
proc smb.getservices {machine type guestflag username password} {
    global smbdir
    
    set servname ""
    set servlist ""
    ## open pipe to smbclient - echo in a null string
    ## in case it asks for a password
    ## just in case the idiot asks for a password
    if {$guestflag} {
	set name "-N"
    } elseif {[string compare $username ""] == 0} {
	set name "-N"
    } else {
	set name "-U $username%$password"
    }
    set info [open "|echo \"\" | $smbdir/smbclient $name -L $machine" r]
    
    while {[gets $info line] != -1} {
	if { [regexp {.*Sharename.*Type.*Comment.*} $line] } {
	    ## eat the visual seperator line
	    ## and prime the next loop
	    gets $info line
	    gets $info line
	    while {![regexp {.*This machine.*} $line]} {
		## idiotic unparseable smbclient formatting
		if {[debug]} {puts "$line"}
		set newtype [string range $line 16 [string wordend $line 22]]
		set newtype [string trim $newtype]
		if {[debug]} {puts "smb: type is $newtype"}
		if {[string compare $type $newtype] == 0} {
		    #regsub "\t(.+)($type)(.*)" $line {\2} servname
		    set servname [string range $line 0 15]
		    set servname [string trim $servname]
		    if {[debug]} {puts "smb: service = $servname; type = $type"}
		    set servlist "$servlist \"$servname\""
		}
		if {[gets $info line] == -1} { break }
	    }
	}
    }
    catch {close $info}

    return $servlist
}

proc smb.getdefzone {} {
    global smbdir env smb_defzone
    set workgroup "workgroup"
    if {[info exists env(HOSTNAME)] == 1} {
	set hostname $env(HOSTNAME)
    } else {
	catch {exec hostname -s} hostname
    }
    if {[debug]} { puts "smb: hostname = $hostname" }
    set info [open "|echo \"\" | $smbdir/smbclient -N -L $hostname" r]
    while {[gets $info line] != -1} {
	if { [regexp {^Server=.*} $line] } {
	    if {[debug]} {puts "smb: $line"}
	    set wgfield [lindex $line 2]
	    regsub {.*Workgroup=\[(.+)\].*} $wgfield {\1} workgroup
	    break
	}
    }
    catch {close $info}
    if {[debug]} {puts "smb: getdefzone: $workgroup"}
    set smb_defzone $workgroup
    return $workgroup
}

## procedure to show current zonelist
proc smb.setzones {} {
    foreach name [smb.getzones] {
	.leftside.bot.zones insert end $name
    }
    update
}

## procedure to set the default zone (at startup)
proc smb.setdefzone {} {
    global smb_defzone smb_currzone
    set thezone $smb_defzone
    set size [.leftside.bot.zones size]
    set smb_currzone $smb_defzone
    for {set i 0} {$i<$size} {incr i} {
	set name [.leftside.bot.zones get $i]
	if {[string compare $name $thezone] == 0} {
	    .leftside.bot.zones selection set $i
	    update
	    return
	}
    }
    
}

## procedure to delete zone listings
proc smb.delzones {} {
    .leftside.bot.zones del 0 end
}

## looping procedure to refresh zone listings
proc smb.refresh {} {
    global smb_zonedelay smb_afterID

    if {[debug]} { puts "smb: refreshing zonelist" }
    smb.delzones
    smb.setzones
    set smb_afterID [after $smb_zonedelay {smb.refresh}]

}

############# End general utility functions ###############

if {[debug]} { puts "Finished loading samba smb module." }





